using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

namespace I2CTest
{
    public class MCP9800_TemperatureSensor
    {
        I2CDevice.Configuration _config;
        I2CDevice _i2c;

        public MCP9800_TemperatureSensor(I2CDevice I2CBus, ushort I2CAddress, int I2CClockFrequencyKHz)
        {
            // Memorizzo la configurazione passata come parametro
            _i2c = I2CBus;
            _config = new I2CDevice.Configuration(I2CAddress, I2CClockFrequencyKHz);
        }

        public MCP9800_TemperatureSensor(I2CDevice I2CBus)
        {
            // Uso la configurazione del bus passato come parametro
            _i2c = I2CBus;
            _config = I2CBus.Config;
        }

        public float Temperature
        {
            get
            {
                // Imposto la configurazione del bus
                _i2c.Config = _config;

                // Creo 2 transazioni, una di scrittura (indice registro da leggere)
                // ed una di lettura (2 byte, contenente la temperatura corrente)
                I2CDevice.I2CTransaction[] tran = new I2CDevice.I2CTransaction[2];

                byte[] tx = new byte[1] { 0 };  // indice registro temperatura=0
                byte[] rx = new byte[2] { 0, 0 };

                tran[0] = _i2c.CreateWriteTransaction(tx);
                tran[1] = _i2c.CreateReadTransaction(rx);

                int retval = _i2c.Execute(tran, 1000);  // Timeout a 1 secondo

                if (retval != tx.Length + rx.Length) throw new ApplicationException("Transazione I2C FALLITA");

                // calcolo temperatura in C
                if ((rx[0] & 0x80) != 0)
                {
                    // se sotto 0C applico il complemento a 2
                    return (~rx[0] * 256 + ~rx[1]) / 256.0f;
                }
                else
                {
                    return (rx[0] * 256 + rx[1]) / 256.0f;
                }
            }
        }
    }
}
