using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

namespace SPITest
{
    public class MCP23S08_IOEXP
    {
        // Indici registri
        const byte REGISTER_IODIR_ADDR = 0x0;
        const byte REGISTER_GP_INT_EN_ADDR = 0x2;
        const byte REGISTER_INT_FLAG_ADDR = 0x7;
        const byte REGISTER_INT_CAP_ADDR = 0x8;
        const byte REGISTER_GPIO_ADDR = 0x9;

        // indirizzo logico sul bus
        const byte MCP23S08_A0 = 0x0;
        const byte MCP23S08_A1 = 0x0;

        // indirizzi logici di lettura e scrittura
        const byte MCP23S08_R_ADDRESS = ((1 << 6) | (MCP23S08_A1 << 2) | (MCP23S08_A0 << 1) | 0x1);
        const byte MCP23S08_W_ADDRESS = ((1 << 6) | (MCP23S08_A1 << 2) | (MCP23S08_A0 << 1) | 0x0);

         // riferimento al bus SPI da utilizzare
        SPI _spi = null;

        // costruttore che accetta in ingresso un bus SPI pre-inizializzato
        public MCP23S08_IOEXP(SPI spi)
        {
            _spi = spi;
        }

        // imposta i GPIO come ingressi (bit=1) o uscite (bit=0)
        public void SetIODirection(byte input_mask)
        {
            byte[] tx = new byte[3] { MCP23S08_W_ADDRESS, REGISTER_IODIR_ADDR, input_mask };
            
            _spi.Write(tx);
        }

        // rileva la direzione (ingresso o uscita) dei GPIO
        public byte GetIODirection()
        {
            byte[] tx = new byte[2] { MCP23S08_R_ADDRESS, REGISTER_IODIR_ADDR };
            byte[] rx = new byte[2] { 0, 0 };

            _spi.WriteRead(tx, rx);

            return rx[1];
        }

        // imposta lo stato dei GPIO (ha un effetto solo per quelli di uscita)
        public void WriteIO(byte output_mask)
        {
            byte[] tx = new byte[3] { MCP23S08_W_ADDRESS, REGISTER_GPIO_ADDR, output_mask };
            
            _spi.Write(tx);
        }

        // rileva lo stato di tutti i GPIO
        public byte ReadIO()
        {
            byte[] tx = new byte[3] { MCP23S08_R_ADDRESS, REGISTER_GPIO_ADDR, 0 };
            byte[] rx = new byte[3] { 0, 0, 0 };

            _spi.WriteRead(tx, rx);

            return rx[2];
        }
    }
}
