﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.Hardware;
using System.IO.Ports;
using System.Text;
using System.Threading;

namespace SPITest
{
    public static class Program
    {
        // parametri configurazione SPI
        static SPI.Configuration _spiConfig = new SPI.Configuration(
            USBizi.Pin.IO43,    // CS Pin
            false,              // CS attivo a LOW
            0,                  // CS setup time
            0,                  // CS hold time
            false,              // SCK idle a LOW
            true,               // campionamento MISO/MOSI su fronti di salita di SCK
            250,                // clock a 250 KHz
            SPI.SPI_module.SPI1);

        // istanza bus SPI
        static SPI _spi = new SPI(_spiConfig);

        // porta seriale
        static SerialPort _uart = new SerialPort("COM1", 9600);

        // driver ADC (MCP3201)
        static MCP3201_ADC _adc = new MCP3201_ADC(_spi);

        // driver IO Expander
        static MCP23S08_IOEXP _ioexp = new MCP23S08_IOEXP(_spi);

        public static void Main()
        {
            string uartstring = null;
            float vadc=0.0f;
            byte outpos = 0;
            byte instate = 0;
            byte ioexpmask = 0xF0;

            _ioexp.SetIODirection(ioexpmask);

            Debug.Assert(_ioexp.GetIODirection() == 0xF0,"IOEXP SetIODirection() FAILED!");

            _uart.Open();

            while (true)
            {
                // ADC
                vadc=_adc.Vin;

                // output seriale
                uartstring = string.Concat(vadc.ToString("F"),"V\r");
                _uart.Write(Encoding.UTF8.GetBytes(uartstring), 0, uartstring.Length);

                // IO Expander
                _ioexp.WriteIO((byte)(1 << outpos++ % 4));

                // stato ingressi
                instate = (byte)((int)_ioexp.ReadIO() >> 4);

                // output seriale
                uartstring = string.Concat(instate.ToString(),"\r");
                _uart.Write(Encoding.UTF8.GetBytes(uartstring), 0, uartstring.Length);

                // debug
                Debug.Print(vadc.ToString("F"));
                Debug.Print(instate.ToString());

                Thread.Sleep(500);
            }
        }

    }
}
