﻿using System;
using Microsoft.SPOT;
using System.IO.Ports;
using System.Text;
using System.Threading;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.Hardware;
using System.Collections;

namespace UARTTest
{
    public static class Program
    {
        // L'istanza di porta seriale da utilizzare
        static SerialPort _uart = new SerialPort("COM1", 9600);

        // Stringhe costanti
        const string SPLASH = "-- UART Test --\r";
        const string END = "-- END --\r";
        const string ERR = "*ERR\r";
        const string OK = "*OK\r";

        // Membri statici (variabili globali dell'applicazione)
        static string _inBuffer = string.Empty;         // buffer contenente i caratteri (filtrati) ricevuti via seriale
        static Queue _commandQueue = new Queue();       // Coda contenente gli ultimi comandi ricevuti

        static object _syncLock = new object();         // Oggetto usato come risorsa di sincronizzazione
        
        static OutputPort _ledPort = new OutputPort((Cpu.Pin)USBizi.Pin.IO2, false);    // Uscita relativa al LED diagnostico

        public static void Main()
        {
            Debug.Print("UART Test starting...");

            // Apertura porta seriale
            _uart.Open();

            // Registrazione evento ricezione dati
            _uart.DataReceived += new SerialDataReceivedEventHandler(_uart_DataReceived);

            // Stringa di "benvenuto"
            writeToUart(SPLASH);

            // Ciclo di processing dei messaggi
            bool exit = false;

            while (!exit)
            {
                // Matching dei comandi riconsciuti
                switch (getLastCommandLocked())
                {
                    case "led on":
                        _ledPort.Write(true);
                        writeToUart(OK);
                        break;

                    case "led off":
                        _ledPort.Write(false);
                        writeToUart(OK);
                        break;

                    case "led toggle":
                        _ledPort.Write(!_ledPort.Read());
                        writeToUart(OK);
                        break;

                    case "exit":
                        exit = true;
                        break;

                    case null:
                        break;

                    default:
                        writeToUart(ERR);
                        break;
                }

                //Thread.Sleep(100);
            }

            // Stringa di "commiato"
            writeToUart(END);

            // Chiusura e deallocazione porta seriale
            _uart.Close();
            _uart.Dispose();

            Debug.Print("...UART Test ended.");

            // Ciclo infinito di attesa reboot
            while (true)
            {
                Thread.Sleep(1000);
            }
        }

        private static void writeToUart(string txt)
        {
            _uart.Write(Encoding.UTF8.GetBytes(txt), 0, txt.Length);
        }

        private static string getLastCommandLocked()
        {
            lock (_syncLock)
            {
                if (_commandQueue.Count == 0) return null;

                return _commandQueue.Dequeue() as string;
            }
        }

        static void _uart_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            if(e.EventType!=SerialData.Chars) return;

            int buflen=_uart.BytesToRead;

            byte[] rxdata=new byte[buflen];
            
            _uart.Read(rxdata,0,buflen);

            byte[] buf = new byte[rxdata.Length];
            int i = 0;

            foreach (byte b in rxdata)
            {   
                if (b >= 48 && b <= 57 || b >= 65 && b <= 90 || b >= 97 && b <= 122 || b==10 || b==13 || b==32)
                {
                    buf[i++] = b;    
                }
            }

            lock (_syncLock)
            {
                _inBuffer+=new string(Encoding.UTF8.GetChars(buf));

                parseCommands();
            }
        }

        private static void parseCommands()
        {
            string[] tokens=_inBuffer.Split('\r');

            if (tokens.Length == 1) return;

            for (int i=0;i<tokens.Length-1;i++)
            {
                _commandQueue.Enqueue(tokens[i]);
            }

            _inBuffer=tokens[tokens.Length-1];
        }

    }
}
